from natural.constant import _, IBAN_ALPHABET
from natural.constant import BBAN_RULES, BBAN_PATTERN, BBAN_MAP
import re


def bban_compact(number):
    '''
    Printable compacted Basic Bank Account Number. Removes all the padding
    characters.

    :param number: string

    >>> bban_compact('1234.56.78.90')
    '1234567890'
    >>> bban_compact('068-9999995-01')
    '068999999501'
    '''
    return re.sub(r'[-. ]', '', str(number))


def bban_base10(number):
    '''
    Printable Basic Bank Account Number in base-10.

    :param number: string

    >>> bban_base10('01234567')
    '45670123'
    >>> bban_base10('ABCD')
    '10111213'
    '''
    number = bban_compact(number)
    number = number[4:] + number[:4]
    return ''.join([str(IBAN_ALPHABET.index(char)) for char in number])


def _bban_regex(structure):
    return re.compile(
        r'^%s$' % BBAN_PATTERN.sub(
            lambda m: '%s{%s}' % (BBAN_MAP[m.group(2)], m.group(1)),
            structure,
        )
    )


def bban(value, country=None, validate=False):
    '''
    Printable Basic Bank Account Number (BBAN) for the given country code. The
    ``country`` must be a valid ISO 3166-2 country code.

    :param value: string or int
    :param country: string

    >>> bban('068-9999995-01', 'BE')
    '068999999501'
    >>> bban('555', 'NL')
    '555'
    >>> bban('555', 'NL', validate=True)
    Traceback (most recent call last):
        ...
    ValueError: Invalid BBAN, number does not match specification
    >>> bban('123', 'XY', validate=True)
    Traceback (most recent call last):
        ...
    ValueError: Invalid BBAN, country unknown
    '''

    value = bban_compact(value)

    if validate:
        country = country.upper()

        try:
            rules = BBAN_RULES[country]
        except KeyError:
            raise ValueError(_('Invalid BBAN, country unknown'))

        regex = _bban_regex(rules['bban'])
        if not regex.match(value):
            raise ValueError(
                _('Invalid BBAN, number does not match specification')
            )

    return value


def iban(number, validate=False):
    '''
    Printable International Bank Account Number (IBAN) as specified in ISO
    13616.

    :param number: string

    >>> iban('BE43068999999501')
    'BE43 0689 9999 9501'
    >>> iban('XY32012341234123', validate=True)
    Traceback (most recent call last):
        ...
    ValueError: Invalid IBAN, country unknown
    >>> iban('BE43068999999502', validate=True)
    Traceback (most recent call last):
        ...
    ValueError: Invalid IBAN, digits check failed

    '''

    number = bban_compact(number)
    if validate:
        country = number[:2]
        if country not in BBAN_RULES:
            raise ValueError(_('Invalid IBAN, country unknown'))

        # Do the 10-mod-97 check
        digits = bban_base10(number)
        if int(digits) % 97 != 1:
            raise ValueError(_('Invalid IBAN, digits check failed'))

        # Check BBAN for country
        bban(number[4:], country, validate=True)

    groups = [number[x:x + 4] for x in range(0, len(number), 4)]
    return ' '.join(groups)



from natural.language import _, _multi
import datetime
import math
import six


# Wed, 02 Oct 2002 08:00:00 EST
# Wed, 02 Oct 2002 13:00:00 GMT
# Wed, 02 Oct 2002 15:00:00 +0200
RFC2822_DATETIME_FORMAT = r'%a, %d %b %Y %T %z'
# Wed, 02 Oct 02 08:00:00 EST
# Wed, 02 Oct 02 13:00:00 GMT
# Wed, 02 Oct 02 15:00:00 +0200
RFC822_DATETIME_FORMAT = r'%a, %d %b %y %T %z'
# 2012-06-13T15:24:17
ISO8601_DATETIME_FORMAT = r'%Y-%m-%dT%H:%M:%S'
# Wed, 02 Oct 2002
RFC2822_DATE_FORMAT = r'%a, %d %b %Y'
# Wed, 02 Oct 02
RFC822_DATE_FORMAT = r'%a, %d %b %y'
# 2012-06-13
ISO8601_DATE_FORMAT = r'%Y-%m-%d'
# All date formats
ALL_DATE_FORMATS = (
    RFC2822_DATE_FORMAT,
    RFC822_DATE_FORMAT,
    ISO8601_DATE_FORMAT,
)
ALL_DATETIME_FORMATS = ALL_DATE_FORMATS + (
    RFC822_DATETIME_FORMAT,
    ISO8601_DATETIME_FORMAT,
)

# Precalculated timestamps
TIME_MINUTE = 60
TIME_HOUR = 3600
TIME_DAY = 86400
TIME_WEEK = 604800


def _total_seconds(t):
    '''
    Takes a `datetime.timedelta` object and returns the delta in seconds.

    >>> _total_seconds(datetime.timedelta(23, 42, 123456))
    1987242
    >>> _total_seconds(datetime.timedelta(23, 42, 654321))
    1987243
    '''
    return sum([
        int(t.days * 86400 + t.seconds),
        int(round(t.microseconds / 1000000.0))
    ])


def _to_datetime(t):
    '''
    Internal function that tries whatever to convert ``t`` into a
    :class:`datetime.datetime` object.


    >>> _to_datetime('2013-12-11')
    datetime.datetime(2013, 12, 11, 0, 0)
    >>> _to_datetime('Wed, 11 Dec 2013')
    datetime.datetime(2013, 12, 11, 0, 0)
    >>> _to_datetime('Wed, 11 Dec 13')
    datetime.datetime(2013, 12, 11, 0, 0)
    >>> _to_datetime('2012-06-13T15:24:17')
    datetime.datetime(2012, 6, 13, 15, 24, 17)

    '''

    if isinstance(t, six.integer_types + (float, )):
        return datetime.datetime.fromtimestamp(t).replace(microsecond=0)

    elif isinstance(t, six.string_types):
        for date_format in ALL_DATETIME_FORMATS:
            try:
                d = datetime.datetime.strptime(t, date_format)
                return d.replace(microsecond=0)
            except ValueError:
                pass

        raise ValueError(_('Format "%s" not supported') % t)

    elif isinstance(t, datetime.datetime):
        return t.replace(microsecond=0)

    elif isinstance(t, datetime.date):
        d = datetime.datetime.combine(t, datetime.time(0, 0))
        return d.replace(microsecond=0)

    else:
        raise TypeError


def _to_date(t):
    '''
    Internal function that tries whatever to convert ``t`` into a
    :class:`datetime.date` object.

    >>> _to_date('2013-12-11')
    datetime.date(2013, 12, 11)
    >>> _to_date('Wed, 11 Dec 2013')
    datetime.date(2013, 12, 11)
    >>> _to_date('Wed, 11 Dec 13')
    datetime.date(2013, 12, 11)
    '''

    if isinstance(t, six.integer_types + (float, )):
        return datetime.date.fromtimestamp(t)

    elif isinstance(t, six.string_types):
        for date_format in ALL_DATE_FORMATS:
            try:
                return datetime.datetime.strptime(t, date_format).date()
            except ValueError:
                pass

        raise ValueError('Format not supported')

    elif isinstance(t, datetime.datetime):
        return t.date()

    elif isinstance(t, datetime.date):
        return t

    else:
        raise TypeError


def delta(t1, t2, words=True, justnow=datetime.timedelta(seconds=10)):
    '''
    Calculates the estimated delta between two time objects in human-readable
    format. Used internally by the :func:`day` and :func:`duration` functions.

    :param t1: timestamp, :class:`datetime.date` or :class:`datetime.datetime`
               object
    :param t2: timestamp, :class:`datetime.date` or :class:`datetime.datetime`
               object
    :param words: default ``True``, allow words like "yesterday", "tomorrow"
               and "just now"
    :param justnow: default ``datetime.timedelta(seconds=10)``,
               :class:`datetime.timedelta` object representing tolerance for
               considering a delta as meaning 'just now'

    >>> (x,y) = delta(_to_datetime('2012-06-13T15:24:17'), \
    _to_datetime('2013-12-11T12:34:56'))
    >>> print(x)
    77 weeks
    >>> int(y)
    -594639
    '''

    t1 = _to_datetime(t1)
    t2 = _to_datetime(t2)
    diff = t1 - t2
    date_diff = t1.date() - t2.date()

    # The datetime module includes milliseconds with float precision. Floats
    # will give unexpected results here, so we round the value here
    total = math.ceil(_total_seconds(diff))
    total_abs = abs(total)

    if total_abs < TIME_DAY:
        if abs(diff) < justnow and words:
            return (
                _('just now'),
                0,
            )

        elif total_abs < TIME_MINUTE:
            seconds = total_abs
            return (
                _multi(
                    _('%d second'),
                    _('%d seconds'),
                    seconds
                ) % (seconds,),
                0,
            )
        elif total_abs < TIME_MINUTE * 2 and words:
            return (
                _('a minute'),
                0,
            )

        elif total_abs < TIME_HOUR:
            minutes, seconds = divmod(total_abs, TIME_MINUTE)
            if total < 0:
                seconds *= -1
            return (
                _multi(
                    _('%d minute'),
                    _('%d minutes'),
                    minutes
                ) % (minutes,),
                seconds,
            )

        elif total_abs < TIME_HOUR * 2 and words:
            return (
                _('an hour'),
                0,
            )

        else:
            hours, seconds = divmod(total_abs, TIME_HOUR)
            if total < 0:
                seconds *= -1

            return (
                _multi(
                    _('%d hour'),
                    _('%d hours'),
                    hours
                ) % (hours,),
                seconds,
            )

    elif date_diff.days == 1 and words:
        return (_('tomorrow'), 0)

    elif date_diff.days == -1 and words:
        return (_('yesterday'), 0)

    elif total_abs < TIME_WEEK:
        days, seconds = divmod(total_abs, TIME_DAY)
        if total < 0:
            seconds *= -1
        return (
            _multi(
                _('%d day'),
                _('%d days'),
                days
            ) % (days,),
            seconds,
        )

    elif abs(diff.days) == TIME_WEEK and words:
        if total > 0:
            return (_('next week'), diff.seconds)
        else:
            return (_('last week'), diff.seconds)

# FIXME
#
# The biggest reliable unit we can supply to the user is a week (for now?),
# because we can not safely determine the amount of days in the covered
# month/year span.

    else:
        weeks, seconds = divmod(total_abs, TIME_WEEK)
        if total < 0:
            seconds *= -1
        return (
            _multi(
                _('%d week'),
                _('%d weeks'),
                weeks
            ) % (weeks,),
            seconds,
        )


def day(t, now=None, format='%B %d'):
    '''
    Date delta compared to ``t``. You can override ``now`` to specify what date
    to compare to.

    You can override the date format by supplying a ``format`` parameter.

    :param t: timestamp, :class:`datetime.date` or :class:`datetime.datetime`
              object
    :param now: default ``None``, optionally a :class:`datetime.datetime`
                object
    :param format: default ``'%B %d'``

    >>> import time
    >>> print(day(time.time()))
    today
    >>> print(day(time.time() - 86400))
    yesterday
    >>> print(day(time.time() - 604800))
    last week
    >>> print(day(time.time() + 86400))
    tomorrow
    >>> print(day(time.time() + 604800))
    next week
    '''
    t1 = _to_date(t)
    t2 = _to_date(now or datetime.datetime.now())
    diff = t1 - t2
    secs = _total_seconds(diff)
    days = abs(diff.days)

    if days == 0:
        return _('today')
    elif days == 1:
        if secs < 0:
            return _('yesterday')
        else:
            return _('tomorrow')
    elif days == 7:
        if secs < 0:
            return _('last week')
        else:
            return _('next week')
    else:
        return t1.strftime(format)


def duration(t, now=None, precision=1, pad=', ', words=None,
             justnow=datetime.timedelta(seconds=10)):
    '''
    Time delta compared to ``t``. You can override ``now`` to specify what time
    to compare to.

    :param t: timestamp, :class:`datetime.date` or :class:`datetime.datetime`
              object
    :param now: default ``None``, optionally a :class:`datetime.datetime`
                object
    :param precision: default ``1``, number of fragments to return
    :param words: default ``None``, allow words like "yesterday", if set to
                  ``None`` this will be enabled if ``precision`` is set to
                  ``1``
    :param justnow: default ``datetime.timedelta(seconds=10)``,
                    :class:`datetime.timedelta` object passed to :func:`delta`
                    representing tolerance for considering argument ``t`` as
                    meaning 'just now'

    >>> import time
    >>> from datetime import datetime
    >>> print(duration(time.time() + 1))
    just now
    >>> print(duration(time.time() + 11))
    11 seconds from now
    >>> print(duration(time.time() - 1))
    just now
    >>> print(duration(time.time() - 11))
    11 seconds ago
    >>> print(duration(time.time() - 3601))
    an hour ago
    >>> print(duration(time.time() - 7201))
    2 hours ago
    >>> print(duration(time.time() - 1234567))
    2 weeks ago
    >>> print(duration(time.time() + 7200, precision=1))
    2 hours from now
    >>> print(duration(time.time() - 1234567, precision=3))
    2 weeks, 6 hours, 56 minutes ago
    >>> print(duration(datetime(2014, 9, 8), now=datetime(2014, 9, 9)))
    yesterday
    >>> print(duration(datetime(2014, 9, 7, 23), now=datetime(2014, 9, 9)))
    1 day ago
    >>> print(duration(datetime(2014, 9, 10), now=datetime(2014, 9, 9)))
    tomorrow
    >>> print(duration(datetime(2014, 9, 11, 1), now=datetime(2014, 9, 9, 23)))
    1 day from now
    '''

    if words is None:
        words = precision == 1

    t1 = _to_datetime(t)
    t2 = _to_datetime(now or datetime.datetime.now())

    if t1 < t2:
        format = _('%s ago')
    else:
        format = _('%s from now')

    result, remains = delta(t1, t2, words=words, justnow=justnow)
    if result in (
            _('just now'),
            _('yesterday'),
            _('tomorrow'),
            _('last week'),
            _('next week'),
    ):
        return result

    elif precision > 1 and remains:
        t3 = t2 - datetime.timedelta(seconds=remains)
        return pad.join([
            result,
            duration(t2, t3, precision - 1, pad, words=False),
        ])

    else:
        return format % (result,)


def compress(t, sign=False, pad=''):
    '''
    Convert the input to compressed format, works with a
    :class:`datetime.timedelta` object or a number that represents the number
    of seconds you want to compress.  If you supply a timestamp or a
    :class:`datetime.datetime` object, it will give the delta relative to the
    current time.

    You can enable showing a sign in front of the compressed format with the
    ``sign`` parameter, the default is not to show signs.

    Optionally, you can chose to pad the output. If you wish your values to be
    separated by spaces, set ``pad`` to ``' '``.

    :param t: seconds or :class:`datetime.timedelta` object
    :param sign: default ``False``
    :param pad: default ``''``

    >>> print(compress(1))
    1s
    >>> print(compress(12))
    12s
    >>> print(compress(123))
    2m3s
    >>> print(compress(1234))
    20m34s
    >>> print(compress(12345))
    3h25m45s
    >>> print(compress(123456))
    1d10h17m36s

    '''

    if isinstance(t, datetime.timedelta):
        seconds = t.seconds + (t.days * 86400)
    elif isinstance(t, six.integer_types + (float, )):
        return compress(datetime.timedelta(seconds=t), sign, pad)
    else:
        return compress(datetime.datetime.now() - _to_datetime(t), sign, pad)

    parts = []
    if sign:
        parts.append('-' if t.days < 0 else '+')

    weeks, seconds = divmod(seconds, TIME_WEEK)
    days, seconds = divmod(seconds, TIME_DAY)
    hours, seconds = divmod(seconds, TIME_HOUR)
    minutes, seconds = divmod(seconds, TIME_MINUTE)

    if weeks:
        parts.append(_('%dw') % (weeks,))
    if days:
        parts.append(_('%dd') % (days,))
    if hours:
        parts.append(_('%dh') % (hours,))
    if minutes:
        parts.append(_('%dm') % (minutes,))
    if seconds:
        parts.append(_('%ds') % (seconds,))

    return pad.join(parts)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

from natural.constant import NATO_ALPHABET, NATO_ALPHABET_PHONETIC
from natural.constant import SPELL_ALPHABET, CODE_PRONOUNCIATION
from natural.constant import CODE_ALPHABET_ARMY, CODE_ALPHABET_FAA
from natural.constant import CODE_ALPHABET_ICAO, CODE_ALPHABET_MORSE
from natural.constant import CODE_ALPHABET_WORD


NATO_ALPHABET_KEYS = sorted(NATO_ALPHABET.keys())


class Spelling(object):
    def __call__(self, sentence, pad=' '):
        parts = []
        for letter in sentence.lower():
            letter = self.transform(letter)
            if letter is not None:
                parts.append(letter)
        return pad.join(parts)

    def transform(self, letter):
        return None


class Alphabet(Spelling):
    '''
    Helper class for (spelling) alphabets.
    '''

    def __init__(self, mapping):
        self.mapping = mapping

    @staticmethod
    def from_pair(self, keys, values):
        '''
        Returns a new :func:`Alphabet` object for the translation items
        specified in ``keys`` and ``values``.
        '''
        return Alphabet(dict(zip(keys, values)))

    def transform(self, letter):
        return self.mapping.get(letter)


ALPHABET = dict(
    code=dict(
        army=Alphabet(CODE_ALPHABET_ARMY[1]),
        faa=Alphabet(CODE_ALPHABET_FAA[1]),
        icao=Alphabet(CODE_ALPHABET_ICAO[1]),
        itu=Alphabet(CODE_ALPHABET_ICAO[1]),
        morse=Alphabet(CODE_ALPHABET_MORSE[1]),
        word=Alphabet(CODE_ALPHABET_WORD[1]),
    ),
    nato=dict(
        telephony=Alphabet(NATO_ALPHABET),
        phonetic=Alphabet(NATO_ALPHABET_PHONETIC),
    ),
    spell=Alphabet(SPELL_ALPHABET),
    pronounce=Alphabet(CODE_PRONOUNCIATION),
)
CODE_PADDING = dict(
    army=CODE_ALPHABET_ARMY[0],
    faa=CODE_ALPHABET_FAA[0],
    icao=CODE_ALPHABET_ICAO[0],
    itu=CODE_ALPHABET_ICAO[0],
    morse=CODE_ALPHABET_MORSE[0],
    word=CODE_ALPHABET_WORD[0],
)


def code(sentence, pad='  ', format='army'):
    '''
    Transform a sentence using the code spelling alphabet, multiple
    international code alphabets are supported.

    ====== ====================================================================
    format description
    ====== ====================================================================
    army   US (international) army code alphabet
    faa    Federal Aviation Administration code alphabet, as described in "ICAO
           Phonetics in the FAA ATC Manual, §2-4-16"
    icao   International Civil Aviation Organization, as described in "Annex 10
           to the Convention on International Civil Aviation, Volume II (Fifth
           edition, 1995), Chapter 5, 38–40"
    itu    International Telecommunication Union Roman alphabet, as described
           in "ITU Phonetic Alphabet and Figure Code"
    morse  International morse code, as described in "International Morse code
           Recommendation ITU-R M.1677-1", http://itu.int/
    word   International Civil Aviation Organization, as described in "Annex 10
           to the Convention on International Civil Aviation, Volume II (Fifth
           edition, 1995), Chapter 5, 38–40"
    ====== ====================================================================

    :param sentence: input sentence
    :param pad: default ``None`` (reside to per-alphabet defaults)
    :param format: default ``army``

    >>> print(code('Python'))
    PAH pah  YANG kee  TANG go  HO tell  OSS car  NOH vem ber
    >>> print(code('Python', format='faa'))
    PAHPAH  YANGKEY  TANGGO  HOHTELL  OSSCAH  NOVEMBER

    >>> print(code('Python', format='icao'))
    PAH PAH  YANG KEY  TANG GO  HOH TELL  OSS CAH  NO VEM BER

    >>> print(code('Python', format='itu'))
    PAH PAH  YANG KEY  TANG GO  HOH TELL  OSS CAH  NO VEM BER

    >>> print(code('Python', format='morse'))
    .--.  -.--  -  ....  ---  -.

    >>> print(code('Python', format='word'))
    papa  yankee  tango  hotel  oscar  november

    '''
    try:
        return ALPHABET['code'][format](sentence, pad or CODE_PADDING[format])
    except KeyError:
        raise TypeError('Unsupported code alphabet "%s"' % (format,))


def morse(sentence, pad=CODE_PADDING['morse']):
    '''
    Wrapper for :func:`code`.

    >>> print(morse('Python'))
    .--. -.-- - .... --- -.

    '''
    return code(sentence, pad, 'morse')


def nato(sentence, pad=' ', format='telephony'):
    '''
    Transform a sentence using the NATO spelling alphabet.

    :param sentence: input sentence
    :param pad: default ``' '``
    :param format: default ``telephony``, options ``telephony`` or ``phonetic``

    >>> print(nato('Python'))
    papa yankee tango hotel oscar november

    >>> print(nato('Python', format='phonetic'))
    pah-pah yang-key tang-go hoh-tel oss-cah no-vem-ber

    '''
    try:
        return '' + ALPHABET['nato'][format](sentence, pad)
    except KeyError:
        raise TypeError('Unsupported NATO alphabet "%s"' % (format,))


def spell(sentence, pad='  '):
    '''
    Transform a sentence using the localised spelling alphabet.

    :param sentence: input sentence
    :param pad: default ``'  '``

    >>> print(spell('Python'))
    Paris  Yokohama  Tripoli  Havanna  Oslo  New York

    '''
    return ALPHABET['spell'](sentence, pad)


def pronounce(sentence, pad=' '):
    '''
    Transform a sentence using the pronounciations of the international code
    spelling alphabet. This function is subject to change its behaviour to
    support internationalised pronounciations of letters.

    :param sentence: input sentence
    :param pad: default ``'  '``


    >>> print(pronounce('abc'))
    ælfɑ ˈbrɑːˈvo ˈtʃɑːli

    '''
    return ALPHABET['pronounce'](sentence, pad)



import hashlib
import re
import six
from binascii import unhexlify
from natural.constant import PHONE_PREFIX, PHONE_E161_ALPHABET
from natural.language import _
from natural.util import luhn_append, luhn_calc, strip, to_decimal


def e123(number, areasize=3, groupsize=4, national=False):
    '''
    Printable E.123 (Notation for national and international telephone numbers
    from ITU) numbers.

    :param number: string
    :param areasize: int
    :param groupsize: int
    :param national: bool

    >>> print(e123(155542315678))
    +1 555 4231 5678
    >>> print(e123('+31654231567', areasize=1))
    +31 6 5423 1567
    >>> print(e123('+3114020', areasize=2))
    +31 14 020
    >>> print(e123('+312054231567', areasize=2, national=True))
    (020) 5423 1567
    '''

    if isinstance(number, six.integer_types):
        return e123('+%s' % number, areasize, groupsize)

    elif isinstance(number, six.string_types):
        number = strip(number, '-. ()')
        if number.startswith('+'):
            number = number[1:]

        if not number.isdigit():
            raise ValueError(_('Invalid telephone number'))

        groups = []
        prefix = ''
        remain = number

        if national:
            for x in six.moves.xrange(3, 0, -1):
                if number[:x] in PHONE_PREFIX:
                    groups.append('(0%s)' % number[x:x + areasize])
                    remain = number[x + areasize:]
                    break

        else:
            prefix = '+'
            for x in six.moves.xrange(3, 0, -1):
                if number[:x] in PHONE_PREFIX:
                    groups.append(number[:x])
                    groups.append(number[x:x + areasize])
                    remain = number[x + areasize:]
                    break

        for x in six.moves.xrange(0, len(remain) + 1, groupsize):
            groups.append(remain[x:x + groupsize])

        return '%s%s' % (prefix, ' '.join(list(filter(None, groups))))


def e161(number, alphabet=PHONE_E161_ALPHABET):
    '''
    Printable a 26 Latin letters (A to Z) phone number to the 12-key telephone
    keypad number.

    :param number: string
    :param alphabet: dict

    >>> print(e161('0800-PIZZA123'))
    080074992123
    >>> e161('0800^PIZZA123')
    Traceback (most recent call last):
        ...
    ValueError: Character "^" (0x5e) is not in the E.161 alphabet
    '''

    digits = []
    for char in strip(number, '+-. ()').lower():
        length = len(digits)
        for group, digit in alphabet.items():
            if char in group:
                digits.append(digit)
                break

        if len(digits) == length:
            raise ValueError(
                _('Character "%s" (0x%02x) is not in the E.161 alphabet') %
                (char, ord(char))
            )

    return u''.join(digits)


def e164(number):
    '''
    Printable E.164 (The international public telecommunication numbering plan
    from ITU) numbers.

    :param number: string

    >>> print(e164(155542315678))
    +155542315678
    >>> print(e164('+31 20 5423 1567'))
    +312054231567
    '''
    if isinstance(number, six.integer_types):
        return e164('+%s' % number)

    elif isinstance(number, six.string_types):
        number = strip(number, '-. ()')
        if number.startswith('+'):
            number = number[1:]

        return u'+%s' % number


def enum(number, zone='e164.arpa'):
    '''
    Printable DNS ENUM (telephone number mapping) record.

    :param number: string
    :param zone: string


    >>> print(enum('+31 20 5423 1567'))
    7.6.5.1.3.2.4.5.0.2.1.3.e164.arpa.
    >>> print(enum('+31 97 99 6642', zone='e164.spacephone.org'))
    2.4.6.6.9.9.7.9.1.3.e164.spacephone.org.

    '''
    number = e164(number).lstrip('+')
    return u'.'.join([
        u'.'.join(number[::-1]),
        zone.strip(u'.'),
        '',
    ])


def imei(number):
    '''
    Printable International Mobile Station Equipment Identity (IMEI) numbers.

    :param number: string or int

    >>> print(imei(12345678901234))
    12-345678-901234-7
    >>> print(imei(1234567890123456))
    12-345678-901234-56
    '''
    number = to_decimal(number)
    length = len(number)
    if length not in (14, 15, 16):
        raise ValueError(
            _('Invaid International Mobile Station Equipment Identity')
        )

    if len(number) == 14:
        # Add Luhn check digit
        number = luhn_append(number)

    groups = (number[:2], number[2:8], number[8:14], number[14:])
    return u'-'.join(list(filter(None, groups)))


def imsi(number):
    '''
    Printable International Mobile Subscriber Identity (IMSI) numbers. Mind
    that there is no validation done on the actual correctness of the MCC/MNC.
    If you wish to validate IMSI numbers, take a look at `python-stdnum`_.

    :param number: string or int

    >>> print(imsi(2042312345))
    204-23-12345

    .. _python-stdnum: https://pypi.python.org/pypi/python-stdnum/
    '''
    number = to_decimal(number)
    groups = (number[:3], number[3:5], number[5:])
    return u'-'.join(list(filter(None, groups)))


def meid(number, separator=u' '):
    '''
    Printable Mobile Equipment Identifier (MEID) number.

    >>> print(meid(123456789012345678))
    1B 69B4BA 630F34 6
    >>> print(meid('1B69B4BA630F34'))
    1B 69B4BA 630F34 6
    '''

    if isinstance(number, six.string_types):
        number = re.sub(r'[\s-]', '', number)

        try:
            number = '%014X' % int(number, 16)
        except ValueError:
            if len(number) < 18 and number.isdigit():
                return meid('%014X' % int(number), separator)
            else:
                raise ValueError(_('Invalid MEID, size mismatch'))
        else:
            if len(number) not in (14, 15):
                raise ValueError(_('Invalid MEID, size mismatch'))

    elif isinstance(number, six.integer_types):
        if number > 0xfffffffffffffff:
                raise ValueError(_('Invalid MEID, size mismatch'))
        return meid(('%014X' % number)[:14], separator)

    else:
        raise TypeError(_('Invalid MEID, input type invalid'))

    number = number.upper()
    region = number[:2]
    manufacturer = number[2:8]
    serial_number = number[8:14]
    check_digit = number[14:]

    if check_digit == '':
        check_digit = luhn_calc(number, chars='0123456789ABCDEF')

    groups = (region, manufacturer, serial_number, check_digit)
    return separator.join(list(filter(None, groups)))


def pesn(number, separator=u''):
    '''
    Printable Pseudo Electronic Serial Number.

    :param number: hexadecimal string

    >>> print(pesn('1B69B4BA630F34E'))
    805F9EF7
    '''

    number = re.sub(r'[\s-]', '', meid(number))
    serial = hashlib.sha1(unhexlify(number[:14]))
    return separator.join(['80', serial.hexdigest()[-6:].upper()])



import re
import six


# Default Luhn check digits (base-10)
DIGITS = '0123456789'


def luhn_checksum(number, chars=DIGITS):
    '''
    Calculates the Luhn checksum for `number`

    :param number: string or int
    :param chars: string

    >>> luhn_checksum(1234)
    4

    '''
    length = len(chars)
    number = [chars.index(n) for n in reversed(str(number))]
    return (
        sum(number[::2]) +
        sum(sum(divmod(i * 2, length)) for i in number[1::2])
    ) % length


def luhn_calc(number, chars=DIGITS):
    '''
    Calculate the Luhn check digit for ``number``.

    :param number: string
    :param chars: string

    >>> luhn_calc('42')
    '2'

    '''

    checksum = luhn_checksum(str(number) + chars[0], chars)
    return chars[-checksum]


def luhn_append(number, chars=DIGITS):
    '''
    Append the Luhn check digit of ``number`` to the input.

    :param number: string
    :param chars: string

    >>> luhn_append('666')
    '6668'

    '''
    return str(number) + luhn_calc(number, chars)


def strip(value, chars):
    '''
    Removes characters in ``chars`` from input ``value``.

    :param value: string
    :param chars: iterable

    >>> strip('hello.world', '.')
    'helloworld'
    '''
    return ''.join([x for x in value if x not in chars])


def to_decimal(number, strip='- '):
    '''
    Converts a number to a string of decimals in base 10.

    >>> to_decimal(123)
    '123'
    >>> to_decimal('o123')
    '83'
    >>> to_decimal('b101010')
    '42'
    >>> to_decimal('0x2a')
    '42'
    '''
    if isinstance(number, six.integer_types):
        return str(number)

    number = str(number)
    number = re.sub(r'[%s]' % re.escape(strip), '', number)

    # hexadecimal
    if number.startswith('0x'):
        return to_decimal(int(number[2:], 16))

    # octal
    elif number.startswith('o'):
        return to_decimal(int(number[1:], 8))

    # binary
    elif number.startswith('b'):
        return to_decimal(int(number[1:], 2))

    else:
        return str(int(number))



from natural.date import duration
from natural.size import filesize
import os
import stat


def accessed(filename):
    '''
    Retrieve how long ago a file has been accessed.

    :param filename: name of the file

    >>> print accessed(__file__)        # doctest: +SKIP
    just now
    '''
    if isinstance(filename, file):
        filename = filename.name

    return duration(os.stat(filename)[stat.ST_ATIME])


def created(filename):
    '''
    Retrieve how long ago a file has been created.

    :param filename: name of the file

    >>> print created('/')             # doctest: +SKIP
    8 weeks ago
    '''
    if isinstance(filename, file):
        filename = filename.name

    return duration(os.stat(filename)[stat.ST_CTIME])


def modified(filename):
    '''
    Retrieve how long ago a file has been modified.

    :param filename: name of the file

    >>> print modified('/')             # doctest: +SKIP
    3 days ago
    '''
    if isinstance(filename, file):
        filename = filename.name

    return duration(os.stat(filename)[stat.ST_MTIME])


def size(filename, format='decimal'):
    '''
    Retrieve the size of a file.

    :param filename: name of the file

    >>> size('/etc/mime.types')         # doctest: +SKIP
    23.70 kB

    '''
    if isinstance(filename, file):
        filename = filename.name

    return filesize(os.stat(filename)[stat.ST_SIZE], format)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import re
import six
from natural.language import _


'''
.. py:attribute:: IBAN_ALPHABET

   Valid characters in an International Bank Account Number (IBAN).


.. py:attribute:: BBAN_RULES

   Per-country rules for Basic Bank Account Numbers (BBAN) and International
   Bank Account Numbers (IBAN), as specified by the Society for Worldwide
   Interbank Financial Telecommunication (SWIFT).


.. py:attribute:: BBAN_PATTERN

   Regular expression that captures a valid BBAN specification pattern.


.. py:attribute:: BBAN_MAP

   Map of BBAN specification pattern character groups.


.. py:attribute:: ORDINAL_SUFFIX

   Suffixes for the ordinal number representation.


.. py:attribute:: LARGE_NUMBER_SUFFIX

   Suffixes for the word number representation.


.. py:attribute:: FILESIZE_SUFFIX

   Suffixes for the file size representations.


.. py:function:: _(message)

   Return the localised translation of ``message``, based on the current global
   domain, language, and locales directory.


.. py:function:: _multi(singular, plural, count)

   Return the localised translation based on ``count``, the ``singular``
   version if ``count == 1``, the ``plural`` version otherwise.

'''


# natural.bank
BBAN_RULES = dict(
    AD=dict(bban='4!n4!n12!c',
            name=_('Andorra')),
    AE=dict(bban='3!n16!n',
            name=_('United Arab Emirates')),
    AL=dict(bban='8!n16!c',
            name=_('Albania')),
    AT=dict(bban='5!n11!n',
            name=_('Austria')),
    AZ=dict(bban='4!a20!c',
            name=_('Republic of Azerbaijan')),
    BA=dict(bban='3!n3!n8!n2!n',
            name=_('Bosnia and Herzegovina')),
    BE=dict(bban='3!n7!n2!n',
            name=_('Belgium')),
    BG=dict(bban='4!a4!n2!n8!c',
            name=_('Bulgaria')),
    BH=dict(bban='4!a14!c',
            name=_('Bahrain (Kingdom of)')),
    BR=dict(bban='8!n5!n10!n1!a1!c',
            name=_('Brazil')),
    CH=dict(bban='5!n12!c',
            name=_('Switzerland')),
    CR=dict(bban='3!n14!n',
            name=_('Costa Rica')),
    CY=dict(bban='3!n5!n16!c',
            name=_('Cyprus')),
    CZ=dict(bban='4!n6!n10!n',
            name=_('Czech Republic')),
    DE=dict(bban='8!n10!n',
            name=_('Germany')),
    DK=dict(bban='4!n9!n1!n',
            name=_('Denmark')),
    FO=dict(bban='4!n9!n1!n',
            name=_('Denmark (Faroe Islands')),
    GL=dict(bban='4!n9!n1!n',
            name=_('Denmark (Greenland)')),
    DO=dict(bban='4!c20!n',
            name=_('Dominican Republic')),
    EE=dict(bban='2!n2!n11!n1!n',
            name=_('Estonia')),
    ES=dict(bban='4!n4!n1!n1!n10!n',
            name=_('Spain')),
    FI=dict(bban='Not in use',
            name=_('Finland')),
    FR=dict(bban='5!n5!n11!c2!n',
            name=_('France')),
    GB=dict(bban='4!a6!n8!n',
            name=_('United Kingdom')),
    GE=dict(bban='2!a16!n',
            name=_('Georgia')),
    GI=dict(bban='4!a15!c',
            name=_('Gibraltar')),
    GR=dict(bban='3!n4!n16!c',
            name=_('Greece')),
    GT=dict(bban='4!c20!c',
            name=_('Guatemala')),
    HR=dict(bban='7!n10!n',
            name=_('Croatia')),
    HU=dict(bban='3!n4!n1!n15!n1!n',
            name=_('Hungary')),
    IE=dict(bban='4!a6!n8!n',
            name=_('Ireland')),
    IL=dict(bban='3!n3!n13!n',
            name=_('Israel')),
    IS=dict(bban='4!n2!n6!n10!n',
            name=_('Iceland')),
    IT=dict(bban='1!a5!n5!n12!c',
            name=_('Italy')),
    KW=dict(bban='4!a22!c',
            name=_('Kuwait')),
    KZ=dict(bban='3!n13!c',
            name=_('Kazakhstan')),
    LB=dict(bban='4!n20!c',
            name=_('Lebanon')),
    LI=dict(bban='5!n12!c',
            name=_('Liechtenstein (Principality of)')),
    LT=dict(bban='5!n11!n',
            name=_('Lithuania')),
    LU=dict(bban='3!n13!c',
            name=_('Luxembourg')),
    LV=dict(bban='4!a13!c',
            name=_('Latvia')),
    MC=dict(bban='5!n5!n11!c2!n',
            name=_('Monaco')),
    MD=dict(bban='2!c18!c',
            name=_('Republic of Moldova')),
    ME=dict(bban='3!n13!n2!n',
            name=_('Montenegro')),
    MK=dict(bban='3!n10!c2!n',
            name=_('Macedonia, Former Yugoslav Republic of')),
    MR=dict(bban='5!n5!n11!n2!n',
            name=_('Mauritania')),
    MT=dict(bban='4!a5!n18!c',
            name=_('Malta')),
    MU=dict(bban='4!a2!n2!n12!n3!n3!a',
            name=_('Mauritius')),
    NL=dict(bban='4!a10!n',
            name=_('The Netherlands')),
    NO=dict(bban='4!n6!n1!n ',
            name=_('Norway')),
    PK=dict(bban='4!a16!c',
            name=_('Pakistan')),
    PL=dict(bban='8!n16!n',
            name=_('Poland')),
    PS=dict(bban='4!a21!c',
            name=_('Palestine, State of')),
    PT=dict(bban='4!n4!n11!n2!n',
            name=_('Portugal')),
    QA=dict(bban='2!n4!a21!c',
            name=_('Qatar')),
    RO=dict(bban='4!a16!c',
            name=_('Romania')),
    RS=dict(bban='3!n13!n2!n',
            name=_('Serbia')),
    SA=dict(bban='2!n18!c',
            name=_('Saudi Arabia')),
    SE=dict(bban='3!n16!n1!n',
            name=_('Sweden')),
    SI=dict(bban='5!n8!n2!n',
            name=_('Slovenia')),
    SK=dict(bban='4!n6!n10!n',
            name=_('Slovak Republic')),
    SM=dict(bban='1!a5!n5!n12!c',
            name=_('San Marino')),
    TN=dict(bban='2!n3!n13!n2!n',
            name=_('Tunisia')),
    TR=dict(bban='5!n1!c16!c',
            name=_('Turkey')),
    VG=dict(bban='4!a16!n',
            name=_('Virgin Islands, British')),
)
BBAN_PATTERN = re.compile(r'([1-9][0-9]*)!([acen])')
BBAN_MAP = dict(
    a=r'[A-Z]',
    c=r'[a-zA-Z0-9]',
    e=r' ',
    n=r'[0-9]',
)
IBAN_ALPHABET = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'


# natural.number
ORDINAL_SUFFIX = (
    _('th'),
    _('st'),
    _('nd'),
    _('rd'),
    _('th'),
    _('th'),
    _('th'),
    _('th'),
    _('th'),
    _('th'),
)
LARGE_NUMBER_SUFFIX = (
    _('thousand'),
    _('million'),
    _('billion'),
    _('trillion'),
    _('quadrillion'),
    _('quintillion'),
    _('sextillion'),
    _('septillion'),
    _('octillion'),
    _('nonillion'),
    _('decillion'),
    _('undecillion'),
    _('duodecillion'),
    _('tredecillion'),
    _('quattuordecillion'),
    _('quindecillion'),
    _('sexdecillion'),
    _('septendecillion'),
    _('octodec'),
    _('novemdecillion'),
    _('vigintillion'),
    _('unvigintillion'),
    _('duovigintil'),
    _('tresvigintillion'),
    _('quattuorvigintillion'),
    _('quinquavigintillion'),
    _('sesvigintillion'),
    _('septemvigintillion'),
    _('octovigintillion'),
    _('novemvigintillion'),
    _('trigintillion'),
    _('untrigintillion'),
    _('duotrigintillion'),
    _('trestrigintillion'),
    _('quattuortrigintillion'),
    _('quinquatrigintillion'),
    _('sestrigintillion'),
    _('septentrigintillion'),
    _('octotrigintillion'),
    _('novemtrigintillion'),
    _('quadragintillion'),
)

# natural.file
FILESIZE_SUFFIX = dict(
    decimal=('B', 'kB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'),
    binary=('iB', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB'),
    gnu='BKMGTPEZY',
)

# natural.data
PRINTABLE = list(map(lambda c: chr(c), six.moves.xrange(0x20, 0x7f)))
SPARKCHAR = '\u2581\u2582\u2583\u2584\u2585\u2586\u2587\u2588'

# natural.phone
PHONE_E161_ALPHABET = {
    '0':     '0',
    '1':     '1',
    '2abc':  '2',
    '3def':  '3',
    '4ghi':  '4',
    '5jkl':  '5',
    '6mno':  '6',
    '7prqs': '7',
    '8tuv':  '8',
    '9xyz':  '9',
    '*':     '*',
    '#':     '#',
}
PHONE_PREFIX = list(filter(None, re.split(r'[\n\s]*', '''
1   20  21  211 212 213 216 218 220 221 222 223 224 225 226 227 228 229 230 231
232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251
252 253 254 255 256 257 258 260 261 262 263 264 265 266 267 268 269 27  290 291
297 298 299 30  31  32  33  34  350 351 352 353 354 355 356 357 358 359 36  370
371 372 373 374 375 376 377 378 379 380 381 382 385 386 387 388 389 39  40  41
420 421 423 43  44  45  46  47  48  49  500 501 502 503 504 505 506 507 508 509
51  52  53  54  55  56  57  58  590 591 592 593 594 595 596 597 598 599 60  61
62  63  64  65  66  670 672 673 674 675 676 677 678 679 680 681 682 683 685 686
687 688 689 690 691 692 7   800 808 81  82  84  850 851 852 853 854 855 856 86
870 871 872 873 874 878 880 881 882 883 886 888 90  91  92  93  94  95  960 961
962 963 964 965 966 967 968 970 971 972 973 974 975 976 977 979 98  992 993 994
995 996 998
''')))

# natural.spelling
SPELL_ALPHABET = {
    'a': _('Amsterdam'),
    'ä': _('ä'),          # There is no en_US entry for this
    'b': _('Baltimore'),
    'c': _('Casablanca'),
    'd': _('Danmark'),
    'e': _('Edison'),
    'f': _('Florida'),
    'g': _('Gallipoli'),
    'h': _('Havanna'),
    'i': _('Italia'),
    'j': _('Jerusalem'),
    'k': _('Kilogramme'),
    'l': _('Liverpool'),
    'm': _('Madagaskar'),
    'n': _('New York'),
    'ñ': _('ñ'),          # There is no en_US entry for this
    'o': _('Oslo'),
    'ö': _('ö'),          # There is no en_US entry for this
    'p': _('Paris'),
    'q': _('Québec'),
    'r': _('Roma'),
    's': _('Santiago'),
    't': _('Tripoli'),
    'u': _('Uppsala'),
    'ü': _('ü'),          # There is no en_US entry for this
    'v': _('Valencia'),
    'w': _('Washington'),
    'x': _('Xantippe'),
    'y': _('Yokohama'),
    'z': _('Zürich'),
}
NATO_ALPHABET = {
    '0': 'zero',
    '1': 'one',
    '2': 'two',
    '3': 'three',
    '4': 'four',
    '5': 'five',
    '6': 'six',
    '7': 'seven',
    '8': 'eight',
    '9': 'nine',
    'a': 'alpha',
    'b': 'bravo',
    'c': 'charlie',
    '-': 'dash',
    'd': 'delta',
    'e': 'echo',
    'f': 'foxtrot',
    'g': 'golf',
    'h': 'hotel',
    'i': 'india',
    'j': 'juliet',
    'k': 'kilo',
    'l': 'lima',
    'm': 'mike',
    'n': 'november',
    'o': 'oscar',
    'p': 'papa',
    'q': 'quebec',
    'r': 'romeo',
    's': 'sierra',
    '.': 'stop',
    't': 'tango',
    'u': 'uniform',
    'v': 'victor',
    'w': 'whiskey',
    'x': 'xray',
    'y': 'yankee',
    'z': 'zulu',
}
NATO_ALPHABET_PHONETIC = {
    '0': 'zee-ro',
    '1': 'wun',
    '2': 'too',
    '3': 'tree',
    '4': 'fow-er',
    '5': 'fife',
    '6': 'six',
    '7': 'sev-en',
    '8': 'ait',
    '9': 'nin-er',
    'a': 'alf-ah',
    'b': 'brah-voh',
    'c': 'char-lee',
    '-': 'das',
    'd': 'dell-tah',
    'e': 'eck-oh',
    'f': 'foks-trot',
    'g': 'golf',
    'h': 'hoh-tel',
    'i': 'in-dee-ah',
    'j': 'jew-lee-ett',
    'k': 'key-loh',
    'l': 'lee-mah',
    'm': 'mike',
    'n': 'no-vem-ber',
    'o': 'oss-cah',
    'p': 'pah-pah',
    'q': 'keh-beck',
    'r': 'row-me-oh',
    's': 'see-air-rah',
    '.': 'stop',
    't': 'tang-go',
    'u': 'you-nee-form',
    'v': 'vic-tah',
    'w': 'wiss-key',
    'x': 'ecks-ray',
    'y': 'yang-key',
    'z': 'zoo-loo',
}
CODE_ALPHABET_ARMY = ('  ', {
    '0': 'ZE roo',
    '1': 'OO nah wun',
    '2': 'BEES soh too',
    '3': 'TAY rah tree',
    '4': 'KAR tay fower',
    '5': 'PAN tah five',
    '6': 'SOK see six',
    '7': 'SAY tay seven',
    '8': 'OK toh eight',
    '9': 'NO vay niner',
    'a': 'AL fah',
    'b': 'BRAH voh',
    'c': 'CHAR lee',
    '-': 'DASH',
    'd': 'DEL tah',
    'e': 'EKK oh',
    'f': 'FOKS trot',
    'g': 'Golf',
    'h': 'HO tell',
    'i': 'IN dee ah',
    'j': 'JEW lee ett',
    'k': 'KEY loh',
    'l': 'LEE mah',
    'm': 'Mike',
    'n': 'NOH vem ber',
    'o': 'OSS car',
    'p': 'PAH pah',
    'q': 'keh BECK',
    'r': 'ROW me oh',
    's': 'see AIR ah',
    '.': 'STOP',
    't': 'TANG go',
    'u': 'YOU nee form',
    'v': 'VIK ter',
    'w': 'WISS key',
    'x': 'EKS ray',
    'y': 'YANG kee',
    'z': 'ZOO loo',
})
CODE_ALPHABET_FAA = (u' ', {
    '0': 'ZERO',
    '1': 'WUN',
    '2': 'TOO',
    '3': 'TREE',
    '4': 'FOW ER',
    '5': 'FIFE',
    '6': 'SIX',
    '7': 'SEV EN',
    '8': 'AIT',
    '9': 'NIN ER',
    'a': 'ALFAH',
    'b': 'BRAHVOH',
    'c': 'CHARLEE',
    'd': 'DELLTAH',
    'e': 'ECKOH',
    'f': 'FOKSTROT',
    'g': 'GOLF',
    'h': 'HOHTELL',
    'i': 'INDEE AH',
    'j': 'JEWLEE ETT',
    'k': 'KEYLOH',
    'l': 'LEEMAH',
    'm': 'MIKE',
    'n': 'NOVEMBER',
    'o': 'OSSCAH',
    'p': 'PAHPAH',
    'q': 'KEHBECK',
    'r': 'ROWME OH',
    's': 'SEEAIRAH',
    '.': 'STOP',
    't': 'TANGGO',
    'u': 'YOUNEE FORM',
    'v': 'VIKTAH',
    'w': 'WISSKEY',
    'x': 'ECKSRAY',
    'y': 'YANGKEY',
    'z': 'ZOOLOO',
})
CODE_ALPHABET_ICAO = ('  ', {
    '0': 'ZE RO',
    '1': 'OO NAH WUN',
    '2': 'TOO',
    '3': 'TREE',
    '4': 'FOW ER',
    '5': 'FIFE',
    '6': 'SIX',
    '7': 'SEVEN',
    '8': 'AIT',
    '9': 'NIN ER',
    'a': 'AL FAH',
    'b': 'BRAH VOH',
    'c': 'CHAR LEE',
    'd': 'DELL TAH',
    'e': 'ECK OH',
    'f': 'FOKS TROT',
    'g': 'GOLF',
    'h': 'HOH TELL',
    'i': 'IN DEE AH',
    'j': 'JEW LEE ETT',
    'k': 'KEY LOH',
    'l': 'LEE MAH',
    'm': 'MIKE',
    'n': 'NO VEM BER',
    'o': 'OSS CAH',
    'p': 'PAH PAH',
    'q': 'KEH BECK',
    'r': 'ROW ME OH',
    's': 'SEE AIR RAH',
    '.': 'STOP',
    't': 'TANG GO',
    'u': 'YOU NEE FORM',
    'v': 'VIK TAH',
    'w': 'WISS KEY',
    'x': 'ECKS RAY',
    'y': 'YANG KEY',
    'z': 'ZOO LOO',
})
CODE_ALPHABET_MORSE = (' ', {
    '=': '-...-',
    ' ': '/',
    '_': '..--.-',
    '-': '-....-',
    ',': '--..--',
    ';': '-.-.-.',
    ':': '---...',
    '!': '-.-.--',
    '?': '..--..',
    '/': '-..-.',
    '.': '.-.-.-',
    '"': '.-..-.',
    '(': '-.--.',
    ')': '-.--.-',
    '$': '..._.._',
    '&': '.-...',
    '+': '.-.-.',
    "'": '.----.',
    '0': '-----',
    '1': '.----',
    '2': '..---',
    '3': '...--',
    '4': '....-',
    '5': '.....',
    '6': '-....',
    '7': '--...',
    '8': '---..',
    '9': '----.',
    'a': '.-',
    '@': '.--.-.',      # A + C
    'b': '-...',
    'c': '-.-.',
    'd': '-..',
    'e': '.',
    'f': '..-.',
    'g': '--.',
    'h': '....',
    'i': '..',
    'j': '.---',
    'k': '-.-',
    'l': '.-..',
    'm': '--',
    'n': '-.',
    'o': '---',
    'p': '.--.',
    'q': '--.-',
    'r': '.-.',
    's': '...',
    't': '-',
    'u': '..-',
    'v': '...-',
    'w': '.--',
    'x': '-..-',
    'y': '-.--',
    'z': '--..',
})
CODE_ALPHABET_WORD = (' ', {
    '0': 'zero',
    '1': 'one',
    '2': 'two',
    '3': 'three',
    '4': 'four',
    '5': 'five',
    '6': 'six',
    '7': 'seven',
    '8': 'eight',
    '9': 'nine',
    'a': 'alfa',
    'b': 'bravo',
    'c': 'charlie',
    '-': 'dash',
    'd': 'delta',
    'e': 'echo',
    'f': 'foxtrot',
    'g': 'golf',
    'h': 'hotel',
    'i': 'india',
    'j': 'juliett',
    'k': 'kilo',
    'l': 'lima',
    'm': 'mike',
    'n': 'november',
    'o': 'oscar',
    'p': 'papa',
    'q': 'quebec',
    'r': 'romeo',
    's': 'sierra',
    '.': 'stop',
    't': 'tango',
    'u': 'uniform',
    'v': 'victor',
    'w': 'whiskey',
    'x': 'x-ray',
    'y': 'yankee',
    'z': 'zulu',
})
CODE_PRONOUNCIATION = {
    '0': _('ˈzɛroʊ'),
    '1': _('ˈwʌn'),
    '2': _('ˈtuː'),
    '3': _('ˈtriː'),
    '4': _('ˈfoʊ.ər'),
    '5': _('ˈfaɪf'),
    '6': _('ˈsɪks'),
    '7': _('ˈsɛvɛn'),
    '8': _('ˈeɪt'),
    '9': _('ˈnaɪnər'),
    'a': _('ælfɑ'),
    'b': _('ˈbrɑːˈvo'),
    'c': _('ˈtʃɑːli'),
    'd': _('ˈdeltɑ '),
    'e': _('ˈeko'),
    'f': _('ˈfɔkstrɔt'),
    'g': _('ɡʌlf'),
    'h': _('hoːˈtel'),
    'i': _('ˈindiˑɑ'),
    'j': _('ˈdʒuːliˑˈet'),
    'k': _('ˈkiːlo'),
    'l': _('ˈliːmɑ'),
    'm': _('mɑik'),
    'n': _('noˈvembə'),
    'o': _('ˈɔskɑ'),
    'p': _('pəˈpɑ'),
    'q': _('keˈbek'),
    'r': _('ˈroːmiˑo'),
    's': _('siˈerɑ'),
    't': _('ˈtænɡo'),
    '.': _('ˈstɒp'),
    'u': _('ˈjuːnifɔːm'),
    'v': _('ˈviktɑ'),
    'w': _('ˈwiski'),
    'x': _('ˈeksˈrei'),
    'y': _('ˈjænki'),
    'z': _('ˈzuːluː'),
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from __future__ import print_function

import datetime
import fcntl
import os
import struct
import sys
import termios
try:
    from io import BytesIO
except ImportError:
    from cStringIO import StringIO as BytesIO
import six
from natural.constant import PRINTABLE, SPARKCHAR
from natural.language import _
from natural.file import filesize


def _termsize():
    '''
    Get the current terminal size, returns a ``(height, width)`` tuple.
    '''

    try:
        return struct.unpack(
            'hh',
            fcntl.ioctl(sys.stdout.fileno(), termios.TIOCGWINSZ, '1234')
        )
    except:
        return (
            int(os.environ.get('LINES', 25)),
            int(os.environ.get('COLUMNS', 80)),
        )


def hexdump(stream):
    '''
    Display stream contents in hexadecimal and ASCII format. The ``stream``
    specified must either be a file-like object that supports the ``read``
    method to receive bytes, or it can be a string.

    To dump a file::

        >>> hexdump(file(filename))     # doctest: +SKIP

    Or to dump stdin::

        >>> import sys
        >>> hexdump(sys.stdin)          # doctest: +SKIP

    :param stream: stream input
    '''

    if isinstance(stream, six.string_types):
        stream = BytesIO(stream)

    row = 0
    while True:
        data = stream.read(16)
        if not data:
            break

        hextets = data.encode('hex').ljust(32)
        canonical = printable(data)

        print('%08x %s  %s  |%s|' % (
            row * 16,
            ' '.join(hextets[x:x + 2] for x in range(0x00, 0x10, 2)),
            ' '.join(hextets[x:x + 2] for x in range(0x10, 0x20, 2)),
            canonical,
        ))
        row += 1


def printable(sequence):
    '''
    Return a printable string from the input ``sequence``

    :param sequence: byte or string sequence

    >>> print(printable('\\x1b[1;34mtest\\x1b[0m'))
    .[1;34mtest.[0m
    >>> printable('\\x00\\x01\\x02\\x03\\x04\\x05\\x06\\x06') == '........'
    True

    >>> print(printable('12345678'))
    12345678
    >>> print(printable('testing\\n'))
    testing.

    '''
    return ''.join(list(
        map(lambda c: c if c in PRINTABLE else '.', sequence)
    ))


def sparkline(data):
    '''
    Return a spark line for the given data set.

    :value data: sequence of numeric values

    >>> print sparkline([1, 2, 3, 4, 5, 6, 5, 4, 3, 1, 5, 6])  # doctest: +SKIP
    ▁▂▃▄▅▆▅▄▃▁▅▆

    '''

    min_value = float(min(data))
    max_value = float(max(data))
    steps = (max_value - min_value) / float(len(SPARKCHAR) - 1)
    return ''.join([
        SPARKCHAR[int((float(value) - min_value) / steps)]
        for value in data
    ])


def throughput(sample, window=1, format='decimal'):
    '''
    Return the throughput in (intelli)bytes per second.

    :param sample: number of samples sent
    :param window: default 1, sample window in seconds or
                   :class:`datetime.timedelta` object
    :param format: default 'decimal', see :func:`natural.size.filesize`

    >>> print(throughput(123456, 42))
    2.87 kB/s
    '''

    if isinstance(window, datetime.timedelta):
        window = float(window.days * 86400 + window.seconds)
    elif isinstance(window, six.string_types):
        window = float(window)

    per_second = sample / float(window)
    return _('%s/s') % (filesize(per_second, format=format),)



# -*- coding: utf-8 -*-
import gettext
import locale
import os
import six


'''
.. py:attribute:: LOCALE_PATH

   Path on the file system to the locale/ directory, used to find translations
   for the current locale.


.. py:attribute:: CONVENTION

   Container for all the locale conventions, see
   http://docs.python.org/library/locale.html#locale.localeconv
'''


locale.setlocale(locale.LC_ALL, '')

LOCALE_PATH = os.path.join(os.path.dirname(__file__), 'locales')
gettext.bindtextdomain('natural', LOCALE_PATH)
gettext.textdomain('natural')
try:
    TRANSLATION = gettext.translation('natural', LOCALE_PATH)
    if six.PY2:
        _ = TRANSLATION.ugettext
    else:
        _ = TRANSLATION.gettext
except IOError:
    if six.PY2:
        _ = gettext.NullTranslations().ugettext
    else:
        _ = gettext.NullTranslations().gettext

# generic
CONVENTION = locale.localeconv()


def _multi(singular, plural, count):
    '''
    Provides translations for plural and singular forms of a term.
    '''
    if count == 1:
        return singular
    else:
        return plural
